#include "CameraHandler.h"

#include <windows.h>

#include <FL/Fl.h>
#include <FL/Gl.h>

#include <GL/glu.h>

#include <cmath>
#include <iostream>

using namespace std;

CameraHandler::CameraHandler(double ir, double itheta, double iphi, double imintheta, 
							 double imaxtheta, double iminphi, double imaxphi,
							 Fl_Widget* iparent) : r(ir), theta_(itheta), phi_(iphi), 
							 minTheta(imintheta), maxTheta(imaxtheta), minPhi(iminphi),
							 maxPhi(imaxphi), parent(iparent), focus(0, 0, 0)
{
}

CameraHandler::~CameraHandler(void)
{
}

void CameraHandler::projection(Point p)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 45, static_cast<double>(parent->w()) / parent->h(), 1, 10000);

	Point::triple pos, dir, up;
	up = p.normal();
	pos = p.position() + 3 * up;
	dir = p.direction();
	dir.normalize();
	pos = pos - 3 * dir;
	
	dir = pos + dir;

	gluLookAt(pos.x, pos.y, pos.z, dir.x, dir.y, dir.z, up.x, up.y, up.z);

}

void CameraHandler::projection()
{

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 45, static_cast<double>(parent->w()) / parent->h(), 1, 10000);

	
	double rphi = phi_ / 360 * 2 * PI;
	double rtheta = theta_ / 360 * 2 * PI;
	double x, y, z;
	z = r * sin (rphi);
	double r2 = r * cos (rphi);
	y = r2 * sin (rtheta);
	x = r2 * cos (rtheta);
	gluLookAt(x + focus.x, y + focus.y, z + focus.z, focus.x, focus.y, focus.z, 0, 0, 1);	
}

int CameraHandler::process(int e)
{
	switch (e)
	{
	case FL_KEYDOWN:
		switch (Fl::event_key()) {
		case '+':
			r *= 1.1;
			return 1;
		case '-':
			r /= 1.1;
			return 1;
		}
		break;
	case FL_ENTER:
		return 1;
	case FL_MOUSEWHEEL:
		r *= pow(1.1, Fl::event_dy());
		return 1;
	case FL_PUSH:
		switch (Fl::event_button())
		{
		case FL_RIGHT_MOUSE:
			rMouseDown = true;
			rMouseX = Fl::event_x();
			rMouseY = Fl::event_y();
			return 1;
		}
		break;

	case FL_DRAG:
		switch (Fl::event_button())
		{
		case FL_RIGHT_MOUSE:
			theta_ += (Fl::event_x() - rMouseX) / parent->w() * 360;
			if (maxTheta < 360 && theta_ > maxTheta)
				theta_ = maxTheta;

			if (minTheta > -360 && theta_ < minTheta)
				theta_ = minTheta;

			theta_ = theta_ >= 360 ? theta_ - 360 : theta_;
			theta_ = theta_ <= -360 ? theta_ + 360 : theta_;

			rMouseX = Fl::event_x();

			phi_ -= (Fl::event_y() - rMouseY) / parent->h() * 360;
			if (maxPhi < 360 && phi_ > maxPhi)
				phi_ = maxPhi;

			if (minPhi > -360 && phi_ < minPhi)
				phi_ = minPhi;

			phi_ = phi_ >= 360 ? phi_ - 360 : phi_;
			phi_ = phi_ <= -360 ? phi_ + 360 : phi_;

			rMouseY = Fl::event_y();
			return 1;
		}

	case FL_RELEASE:
		switch (Fl::event_button())
		{
		case FL_RIGHT_MOUSE:
			rMouseDown = false;
			return 1;

		}
	}

	return 0;
}



const double CameraHandler::PI = 3.1415927;