#include "Coaster.h"

#include "GroundPlane.h"
#include "train.h"
#include "Config.h"

#include <windows.h>
#include <Fl/Gl.h>

#include <stack>

using namespace std;

Coaster::Coaster(void)
{
	ControlPoint cp;

	curve.addControl(cp);

	cp.x(50);
	cp.y(50);
	cp.z(120);

	curve.addControl(cp);

	cp.x(70);
	cp.y(-50);
	cp.z(15);

	curve.addControl(cp);

	cp.x(-50);
	cp.y(-50);
	cp.z(30);
	
	curve.addControl(cp);

	cp.x(-50);
	cp.y(50);
	cp.z(30);
	
	curve.addControl(cp);

	Renderable::add(&train_);

}

Coaster::~Coaster(void)
{
}

void Coaster::drawThis()
{
	Config& cfg = Config::instance();
	if (cfg.draw_controls())
		curve.drawControls();
	if (cfg.draw_normals())
		curve.drawNormals();


	vector<Point> track_points;
	Point::triple pos, up, out, dir;
		
		triple left, right, left_tie, right_tie;
		
		if (Config::instance().change()) {
			track_points = curve.curve_points();
			left_rail_.clear();
			right_rail_.clear();
			rail_ties_.clear();
			up_dir_.clear();
			out_.clear();
			for (vector<Point>::iterator iter = track_points.begin(); iter != track_points.end();++iter) {
				pos = iter->position();
				up = iter->normal();
				dir = iter->direction();
				out = dir.rotate(up, 90);
				left = triple(pos.x + 2 * out.x, pos.y + 2 * out.y, pos.z + 2 * out.z);
				right = triple(pos.x - 2 * out.x, pos.y - 2 * out.y, pos.z - 2 * out.z);
				
				left_tie = triple(pos.x + 2.5 * out.x - 0.1 * up.x, pos.y + 2.5 * out.y - 0.1 * up.y, pos.z + 2.5 * out.z - 0.1 * up.z);
				right_tie = triple(pos.x - 2.5 * out.x - 0.1 * up.x, pos.y - 2.5 * out.y - 0.1 * up.y, pos.z - 2.5 * out.z - 0.1 * up.z);

				left_rail_.push_back(left);
				right_rail_.push_back(right);	
				rail_ties_.push_back(left_tie);
				rail_ties_.push_back(right_tie);

				up_dir_.push_back(pair<triple, triple>(up, dir));
				out_.push_back(out);
			}

			cfg.change(0);
		}

		

		triple temp1, temp2;
		vector<triple>::iterator i, out_i;
		vector<pair<triple, triple> >::iterator iter;


		triple v1, v2, v3, v4, v5, v6, v7, v8;
	

		iter = up_dir_.end() - 1;
		out_i = out_.end() - 1;
		i = left_rail_.end() - 1;

		v1 = *i + 0.5 * iter->first + 0.1 * (*out_i);
		v2 = *i + 0.5 * iter->first - 0.1 * (*out_i);
		v3 = *i - 0.5 * iter->first - 0.1 * (*out_i);
		v4 = *i - 0.5 * iter->first + 0.1 * (*out_i);

		glBegin(GL_QUADS);
		glColor3f(0.1,0.1,0.1);
		for (i = left_rail_.begin(), iter = up_dir_.begin(), out_i = out_.begin(); i != left_rail_.end();++iter, ++out_i, ++i) {
			v5 = *i + 0.5 * iter->first + 0.1 * (*out_i);
			v6 = *i + 0.5 * iter->first - 0.1 * (*out_i);
			v7 = *i - 0.5 * iter->first - 0.1 * (*out_i);
			v8 = *i - 0.5 * iter->first + 0.1 * (*out_i);

				v5.draw();
				v1.draw();
				v2.draw();
				v6.draw();
				
				v5.draw();
				v8.draw();
				v4.draw();
				v1.draw();

				v6.draw();
				v2.draw();
				v3.draw();
				v7.draw();

				v7.draw();
				v3.draw();
				v4.draw();
				v8.draw();

			v1 = v5;
			v2 = v6;
			v3 = v7;
			v4 = v8;				
			
		}
		glEnd();

		iter = up_dir_.end() - 1;
		out_i = out_.end() - 1;
		i = right_rail_.end() - 1;

		v1 = *i + 0.5 * iter->first + 0.1 * (*out_i);
		v2 = *i + 0.5 * iter->first - 0.1 * (*out_i);
		v3 = *i - 0.5 * iter->first - 0.1 * (*out_i);
		v4 = *i - 0.5 * iter->first + 0.1 * (*out_i);

		glBegin(GL_QUADS);
		glColor3f(0.1,0.1,0.1);
		for (i = right_rail_.begin(), iter = up_dir_.begin(), out_i = out_.begin(); i != right_rail_.end();++iter, ++out_i, ++i) {
			v5 = *i + 0.5 * iter->first + 0.1 * (*out_i);
			v6 = *i + 0.5 * iter->first - 0.1 * (*out_i);
			v7 = *i - 0.5 * iter->first - 0.1 * (*out_i);
			v8 = *i - 0.5 * iter->first + 0.1 * (*out_i);

				v5.draw();
				v1.draw();
				v2.draw();
				v6.draw();
				
				v5.draw();
				v8.draw();
				v4.draw();
				v1.draw();

				v6.draw();
				v2.draw();
				v3.draw();
				v7.draw();

				v7.draw();
				v3.draw();
				v4.draw();
				v8.draw();

			v1 = v5;
			v2 = v6;
			v3 = v7;
			v4 = v8;				
			
		}
		glEnd();

		
		glColor3f(0.5,0.4,0.3);
		for (i = rail_ties_.begin(), iter = up_dir_.begin(); i != rail_ties_.end(); ++i, ++iter) {
		
			v1 = *i + 0.1 * iter->first + 0.4 * iter->second;
			v2 = *i - 0.7 * iter->first + 0.4 * iter->second;
			v3 = *i - 0.7 * iter->first - 0.4 * iter->second;
			v4 = *i + 0.1 * iter->first - 0.4 * iter->second;
			++i;
			v5 = *i + 0.1 * iter->first + 0.4 * iter->second;
			v6 = *i + 0.1 * iter->first - 0.4 * iter->second;
			v7 = *i - 0.7 * iter->first - 0.4 * iter->second;
			v8 = *i - 0.7 * iter->first + 0.4 * iter->second;

			glBegin(GL_QUADS);
			
				glVertex3f(v1.x, v1.y, v1.z);
				glVertex3f(v2.x, v2.y, v2.z);
				glVertex3f(v3.x, v3.y, v3.z);
				glVertex3f(v4.x, v4.y, v4.z);

				glVertex3f(v5.x, v5.y, v5.z);
				glVertex3f(v6.x, v6.y, v6.z);
				glVertex3f(v7.x, v7.y, v7.z);
				glVertex3f(v8.x, v8.y, v8.z);

				glVertex3f(v1.x, v1.y, v1.z);
				glVertex3f(v4.x, v4.y, v4.z);
				glVertex3f(v6.x, v6.y, v6.z);
				glVertex3f(v5.x, v5.y, v5.z);

				glVertex3f(v1.x, v1.y, v1.z);
				glVertex3f(v5.x, v5.y, v5.z);
				glVertex3f(v8.x, v8.y, v8.z);
				glVertex3f(v2.x, v2.y, v2.z);

				v2.draw();
				v8.draw();
				v7.draw();
				v3.draw();

				
				glVertex3f(v3.x, v3.y, v3.z);
				glVertex3f(v7.x, v7.y, v7.z);
				glVertex3f(v6.x, v6.y, v6.z);
				glVertex3f(v4.x, v4.y, v4.z);

			glEnd();

		
	}
}
