#include "CoasterSideView.h"
#include "Config.h"

#include <windows.h>

#include <FL/Fl.h>
#include <FL/Gl.h>

#include <GL/glu.h>

#include <iostream>

CoasterSideView::CoasterSideView(int x, int y, int w, int h, char* t) : Fl_Gl_Window(x, y, w, h, t),
camera(500, 45, 10, -400, 400, 2, 30, this), selector(this), cfg(Config::instance())
{
	FltkListener::add(&camera);
	FltkListener::add(&selector);
}


CoasterSideView::~CoasterSideView(void)
{
}


void CoasterSideView::draw()
{
	if (!valid())
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glViewport(0, 0, w(), h());		
	}

	glClearColor(1, 1, 1, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);

	glEnable (GL_CULL_FACE);
	glCullFace (GL_BACK);

	camera.projection();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	selector.update();

	cfg.world()->draw();
}


int CoasterSideView::process(int e)
{
	
	switch (e)
	{
	case FL_FOCUS:
		return 1;
		
	case FL_UNFOCUS:
		return 1;
	

	case FL_PUSH:
		switch (Fl::event_button())
		{
		case FL_LEFT_MOUSE:
			return 1;
		}

	case FL_DRAG:
		if (Fl::event_button() != FL_LEFT_MOUSE)
			return 0;
		ControlPoint* cp = Config::instance().point();
		if (!cp)
			return 1;
		
		double z = 0;
		selector.select_z(camera.theta(), Fl::event_x(), Fl::event_y(), cp->x(), cp->y(), z);
		cp->z(z);

		return 1;
	}

	return Fl_Gl_Window::handle(e);
}

