#include "CoasterWindow.h"
#include "ControlPoint.h"
#include "Config.h"

#include <windows.h>

#include <FL/Fl.h>
#include <FL/Gl.h>

#include <GL/glu.h>

#include <iostream>
#include <cmath>

using namespace std;

CoasterWindow::CoasterWindow(int x, int y, int w, int h, char* t) : Fl_Gl_Window(x, y, w, h, t),
camera(1000, 45, 45, -400, 400, 5, 85, this), selector(this), cfg(Config::instance())
{
	FltkListener::add(&camera);
	FltkListener::add(&selector);
}

CoasterWindow::~CoasterWindow(void)
{
}

void CoasterWindow::draw()
{
	if (!valid())
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glViewport(0, 0, w(), h());		
	}

	glClearColor(1, 1, 1, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);

	glEnable (GL_CULL_FACE);
	glCullFace (GL_BACK);

	camera.focus_point(cfg.focus());

	if (cfg.riding())
		camera.projection(cfg.train()->pos());
	else
		camera.projection();
		
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	selector.update();

	cfg.world()->draw();
}

int CoasterWindow::process(int e)
{
	
	switch (e)
	{
	case FL_FOCUS:
		return 1;
		
	case FL_UNFOCUS:
		return 1;


	case FL_PUSH:
		switch (Fl::event_button())
		{
		case FL_LEFT_MOUSE:
			return 1;
		}

	case FL_DRAG:
		if (Fl::event_button() != FL_LEFT_MOUSE)
			return 0;
		ControlPoint* cp = Config::instance().point();
		if (!cp)
			return 1;
		
		double x = 0, y = 0;
		selector.select_z_plane(Fl::event_x(), Fl::event_y(), cp->z(), x, y);
		cp->x(x);
		cp->y(y);
	

		return 1;
	}

	return Fl_Gl_Window::handle(e);
}
