#include "Config.h"
#include "ControlPoint.h"

typedef Point::triple triple;

using namespace std;


Config::Config(void) : ready_(false), change_(true), force_normals(false), num_cars(2)
{
}

Config::~Config(void)
{
}

void Config::add_normal() {curve()->add_normal();}

bool Config::riding() {return ui_->ride_train->value();}

bool Config::draw_axes() {return ui_->draw_axes->value();}
bool Config::draw_controls() {return ui_->draw_controls->value();}
bool Config::draw_supports() {return true;}
bool Config::draw_scenery() {return ui_->draw_scenery->value();}
bool Config::draw_normals() {return force_normals || ui_->draw_normals->value();}


void Config::write_curve(std::string s) {if (s.size() < 2) return; curve()->write(s);}
void Config::read_curve(std::string s) {if (s.size() < 2) return; curve()->read(s); change_ = true;}

vector<double> Config::mat(triple dir, triple out, triple up)
{
	double mat_[16] = {
		dir.x, dir.y, dir.z, 0,
		out.x, out.y, out.z, 0,
		up.x,  up.y,  up.z,  0,
		0, 0, 0, 1};

	vector<double> mat;
	for (int i = 0; i < 16; i++)
		mat.push_back(mat_[i]);

	return mat;
}

Config* Config::cfg = 0;

Coaster* Config::coaster() {return cw_->coaster();}
Curve* Config::curve() {return coaster()->getCurve();}
ControlPoint* Config::point() {return curve()->current();}
train* Config::train() {return coaster()->train();}


void Config::next_norm() {instance().curve()->next_norm(); instance().refresh_data();}
void Config::previous_norm() {instance().curve()->previous_norm(); instance().refresh_data();}

int Config::spline() {
	return ui_->spline_type_->value();
}

double Config::speed() {
	return ui_->speed_->value();
}

Point::triple Config::focus() {
	int focus = ui_->focus_->value();
	switch (focus) {
		case 1: // normal
			return curve()->normal_pos();
		case 2: // control point
			if (curve()->valid())
			{
				ControlPoint cp = *point();
				return Point::triple(cp.x(), cp.y(), cp.z());
			}
		case 3:
			return Point::triple(0, 0, 0);
		case 4:
			return train()->pos().position();
		default:
			return Point::triple(0, 0, 0);
	}

}

bool Config::animate() {
	return ui_->animate_button_->value();
}

void Config::new_point()
{
	ControlPoint cp;
	cp.x(10);
	cp.y(10);
	cp.z(10);
	curve()->addControl(cp);
}

void Config::remove_point() {
	curve()->remove_point();
}

void Config::refresh_data()
{
	if (!ready_ || !ui_ || !cw_)
		return;
	
	if (!curve()->valid())
	{
		ui_->point_x_->value(-1);
		ui_->point_y_->value(-1);
		ui_->point_z_->value(-1);
	} else {
		ControlPoint* cp = point();
		ui_->point_x_->value(cp->x());
		ui_->point_y_->value(cp->y());
		ui_->point_z_->value(cp->z());
	}
}

