#pragma once
#include "Curve.h"
#include "CoasterUI.h"
#include "ControlPoint.h"
#include "CoasterWorld.h"
#include "Point.h"
#include "train.h"

#include <iostream>
#include <vector>
#include <string>

class CoasterWorld;
class CoasterUI;

class Config
{
private:
	Config(void);
	static Config* cfg;

public:
	static Config& instance() {if (!cfg) cfg = new Config(); return *cfg;}

private:
	CoasterWorld* cw_;
	CoasterUI* ui_;
	bool ready_;
	bool change_;
	double curve_param_;

	int num_cars;

public:
	int cars() {return num_cars;}

	void add_car() {num_cars++;}
	void remove_car() {num_cars--; num_cars = num_cars < 0 ? 0 : num_cars;}

	void add_normal();

	bool force_normals;

	static void next_norm();
	static void previous_norm();

	bool draw_normals();
	bool draw_controls();

	bool riding();

	bool draw_axes();

	void write_curve(std::string);
	void read_curve(std::string);

	bool change() {return change_;}
	void change(int a) {change_ = a;}

	bool draw_supports();
	bool draw_scenery();

	double speed();

	Point::triple focus();

	static std::vector<double> mat(Point::triple, Point::triple, Point::triple);

	double gravity() {return 0.01;}
	double friction() {return 1;}

	void world(CoasterWorld* cw) {cw_ = cw;}
	void ui(CoasterUI* ui) {ui_ = ui;}


	CoasterUI* ui() {return ui_;}

	void ready() {ready_ = true;}
	
	CoasterWorld* world() {return cw_;}
	Coaster* coaster() ;
	Curve* curve() ;
	ControlPoint* point() ;
	train* train();

	void new_point();
	void remove_point();

	void refresh_data();

	bool animate();

	int spline();

	~Config(void);

	void update();
};
