#include "ControlPoint.h"
#include "Config.h"

#include <windows.h>
#include <FL/Gl.h>

#include <iostream>

using namespace std;

ControlPoint::ControlPoint(void) : radius(10), isSelected(false), isHovered(false), rot(0)
{
	pos.push_back(0);
	pos.push_back(0);
	pos.push_back(0);
}

ControlPoint::~ControlPoint(void)
{
}

void ControlPoint::update(int time)
{
	rot+= 1;
}

void ControlPoint::drawThis()
{
	static float v[24] = {-1,  1,  1,
						  -1, -1,  1,
						   1,  1,  1,
						   1, -1,  1,						  
						   1,  1, -1,
						   1, -1, -1,
						  -1,  1, -1,
						  -1, -1, -1 };
	glTranslatef(pos[0], pos[1], pos[2]);
	glScalef(radius/2, radius/2, radius/2);

	glRotatef(rot, 1, 1, 0);

	glPolygonMode(GL_FRONT, GL_FILL);	
	
	glColor3f(0, 0.7, 0.9);

	if (isHovered)
		glColor3f(0.4, 0.2, 0.7);
	
	if (isSelected)
		glColor3f(0.9, 0.2, 0.1);


	glBegin(GL_QUAD_STRIP);
		glVertex3fv(v);
		glVertex3fv(v + 3);
		glVertex3fv(v + 6);
		glVertex3fv(v + 9);
		glVertex3fv(v + 12);
		glVertex3fv(v + 15);
		glVertex3fv(v + 18);
		glVertex3fv(v + 21);
	glEnd();
	glBegin(GL_QUAD_STRIP);
		glVertex3fv(v + 6);
		glVertex3fv(v + 12);
		glVertex3fv(v);
		glVertex3fv(v + 18);
		glVertex3fv(v + 3);
		glVertex3fv(v + 21);
		glVertex3fv(v + 9);
		glVertex3fv(v + 15);
	glEnd();

	glBegin(GL_QUAD_STRIP);
		glVertex3fv(v);
		glVertex3fv(v + 3);
		glVertex3fv(v + 6);
		glVertex3fv(v + 9);
		glVertex3fv(v + 12);
		glVertex3fv(v + 15);
		glVertex3fv(v + 18);
		glVertex3fv(v + 21);
	glEnd();
	glBegin(GL_QUAD_STRIP);
		glVertex3fv(v + 6);
		glVertex3fv(v + 12);
		glVertex3fv(v);
		glVertex3fv(v + 18);
		glVertex3fv(v + 3);
		glVertex3fv(v + 21);
		glVertex3fv(v + 9);
		glVertex3fv(v + 15);
	glEnd();

	glColor3f(0, 0, 0);

	glBegin(GL_LINE_STRIP);
		glVertex3fv(v + 6);
		glVertex3fv(v);
		glVertex3fv(v + 3);
		glVertex3fv(v + 9);
		glVertex3fv(v + 6);
		glVertex3fv(v + 12);
		glVertex3fv(v + 15);
		glVertex3fv(v + 9);
		
	glEnd();

	
	glBegin(GL_LINES);
		glVertex3fv(v + 18);
		glVertex3fv(v + 21);
		glVertex3fv(v + 18);
		glVertex3fv(v);
		glVertex3fv(v + 18);
		glVertex3fv(v + 12);
		glVertex3fv(v + 21);
		glVertex3fv(v + 15);
		glVertex3fv(v + 21);
		glVertex3fv(v + 3);
	glEnd();	

}

void ControlPoint::select()
{
	isSelected = true;
}

void ControlPoint::deselect()
{
	isSelected = false;
}

void ControlPoint::hover()
{
	isHovered = true;
}

void ControlPoint::deHover()
{
	isHovered = false;
}



bool ControlPoint::pick(double p[3], double v1[3])
{
	double e = 0.005;
	
	double v2[3];

	v2[0] = p[0] - pos[0];
	v2[1] = p[1] - pos[1];
	v2[2] = p[2] - pos[2];

	double v1xv2[3];

	v1xv2[0] = v1[1] * v2[2] - v1[2] * v2[1];
	v1xv2[1] = v1[2] * v2[0] - v1[0] * v2[2];
	v1xv2[2] = v1[0] * v2[1] - v1[1] * v2[0];

	double l1 = v1[0] * v1[0] + v1[1] * v1[1] + v1[2] * v1[2];
	double l2 = v1xv2[0] * v1xv2[0] + v1xv2[1] * v1xv2[1] + v1xv2[2] * v1xv2[2];

	if (l2 / l1 < 1.5 * radius * radius + e)
	{
		return true;
	}

	return false;
}

double ControlPoint::squaredDistance(double p[3])
{
	double dx, dy, dz;
	dx = p[0] - pos[0];
	dy = p[1] - pos[1];
	dz = p[2] - pos[2];
	return (dx * dx + dy * dy + dz * dz);
}