#pragma once

#include "ControlPoint.h"
#include "Point.h"

#include <vector>
#include <string>

class Curve
{
public:
	enum {
		B_SPLINE,
		OH,
		CARDINAL
	};

private:
	std::vector<ControlPoint> controls;
	std::vector<ControlPoint>::iterator curr;
	bool valid_;

	std::vector<Point> points_;
	std::vector<double> length_;

	std::vector<std::pair<double, double> > normals;
	int current_norm;

	

	static double segment_length_;
	static double eps;

	void set(std::vector<ControlPoint>::iterator);
	void unset();

	std::vector<Point> b_spline();
	std::vector<Point> cardinal();
	
	std::vector<Point> make_curve(double, double[16]);

	double magnitude(double, double, double);

public:
	Curve(void);
	~Curve(void);

	void next_norm();
	void previous_norm();

	void write(std::string);
	void read(std::string);


	Point::triple normal_pos() 
	{if (normals.size()) 
			return simple_point_at_length(normals[current_norm % normals.size()].first).position(); 
	return Point::triple(0,0,0);
	}
	double* normal() 
	{
		if (normals.size()) 
			return &normals[current_norm % normals.size()].second; 
		return NULL;
	}

	Point::triple normal_dir()
	{
		if (normals.size()) 
			return simple_point_at_length(normals[current_norm % normals.size()].first).direction(); 
		return Point::triple(0,0,0);
	}
	ControlPoint* current() { if (!valid_) return NULL; return &*curr;}
	bool valid() {return valid_;}

	void addControl(ControlPoint c);

	void remove_point();
	void drawControls();

	void drawNormals();

	void add_normal();


	std::vector<Point> curve_points();
	double length() { return length_.back();}

	Point simple_point_at_length(double);
	Point point_at_length(double);

	bool pick(double[3], double[3], ControlPoint*&);
	bool hover(double[3], double[3]);
	bool select(double[3], double[3]);


};


