#include "Point.h"

#include <cmath>
#include <iostream>

using namespace std;


Point::triple operator* (const double  s, const Point::triple& t) {return Point::triple(t.x * s, t.y * s, t.z * s);}

Point::Point(void)
{
}

Point::~Point(void)
{
}

Point::triple Point::triple::interpolate(double alpha, triple p1, triple p2)
{
	double x, y, z;
	x = alpha * p1.x + (1 - alpha) * p2.x;
	y = alpha * p1.y + (1 - alpha) * p2.y;
	z = alpha * p1.z + (1 - alpha) * p2.z;

	return triple(x,y,z);


}

Point::triple Point::triple::rotate(Point::triple axis, double angle)
{
	double c, s, C;
	double convert = 3.1415927 / 180;
	c = cos(convert * angle);
	s = sin(convert * angle);
	C = 1 - c;
	double mat[9] = {
		axis.x * axis.x * C + c,   axis.x * axis.y * C - axis.z * s,   axis.z * axis.x * C + axis.y * s,
		axis.x * axis.y * C + axis.z * s,   axis.y * axis.y * C + c,   axis.y * axis.z * C - axis.x * s,
		axis.z * axis.x * C - axis.y * s,   axis.y * axis.z * C + axis.x * s,   axis.z * axis.z * C + c
	};

	double nx, ny, nz;
	nx = mat[0] * x + mat[1] * y + mat[2] * z;
	ny = mat[3] * x + mat[4] * y + mat[5] * z;
	nz = mat[6] * x + mat[7] * y + mat[8] * z;

	return triple(nx, ny, nz);

}

void Point::triple::normalize() {
	double mult = x * x + y * y + z * z;

	if (mult == 0)
	{
		x = 0;
		y = 0;
		z = 1;
		return;
	}

	mult = 1.0 / sqrt(mult);	

	x *= mult;
	y *= mult;
	z *= mult;

}

void Point::normalize_direction() {
	double mult = dir.x * dir.x + dir.y * dir.y + dir.z * dir.z;
	mult = 1.0 / sqrt(mult);

	if (mult == 0)
	{
		dir.x = 1;
		dir.y = 0;
		dir.z = 0;
		return;
	}

	dir.x *= mult;
	dir.y *= mult;
	dir.z *= mult;

}

void Point::normalize_normal() {
	double mult = norm.x * norm.x + norm.y * norm.y + norm.z * norm.z;
	mult = 1.0 / sqrt(mult);

	if (mult == 0)
	{
		norm.x = 0;
		norm.y = 0;
		norm.z = 1;
		return;
	}

	norm.x *= mult;
	norm.y *= mult;
	norm.z *= mult;

}

pair<Point::triple, Point::triple> Point::up_out()
{

	triple up, out;
	
	out = cross(dir, norm);
	out.normalize();

	up = cross(out, dir);
	up.normalize();

	return pair<triple, triple>(up, out);
}

Point::triple Point::cross(Point::triple b, Point::triple a)
{
	double x, y, z;
	x = a.y * b.z - a.z * b.y;
	y = a.z * b.x - a.x * b.z;
	z = a.x * b.y - a.y * b.x;
	return triple(x,y,z);
}

