#include "PointSelector.h"
#include "Config.h"

#include <windows.h>

#include <Fl/Fl.h>
#include <Fl/Gl.h>

#include <Gl/Glu.h>

#include <iostream>
#include <cmath>

using namespace std;


PointSelector::~PointSelector(void)
{
}

void PointSelector::update()
{
	glGetDoublev(GL_MODELVIEW_MATRIX, model);				
	glGetDoublev(GL_PROJECTION_MATRIX, proj);				
	glGetIntegerv(GL_VIEWPORT, vp);
}

void PointSelector::select_z_plane(double mouse_x, double mouse_y, double z, double& x, double& y)
{
	double p[3];
	double d[3];
	double wx, wy;
	wx = mouse_x;
	wy = parent->h() - mouse_y;
	gluUnProject(wx, wy, 0, model, proj, vp, p, p + 1, p + 2);
	gluUnProject(wx, wy, 1, model, proj, vp, d, d + 1, d + 2);

	d[0] -= p[0];
	d[1] -= p[1];
	d[2] -= p[2];

	double t = (z - p[2]) / d[2];
	x = p[0] + t * d[0];
	y = p[1] + t * d[1];

}

void PointSelector::select_norm_dir(double mouse_x, double mouse_y, 
									double px, double py, double pz, 
									double nx, double ny, double nz, 
									double& hit_x, double& hit_y, double& hit_z)
{
	double p[3];
	double d[3];
	double wx, wy;
	wx = mouse_x;
	wy = parent->h() - mouse_y;
	gluUnProject(wx, wy, 0, model, proj, vp, p, p + 1, p + 2);
	gluUnProject(wx, wy, 1, model, proj, vp, d, d + 1, d + 2);

	d[0] -= p[0];
	d[1] -= p[1];
	d[2] -= p[2];

	double t = (px - p[0]) * nx + (py - p[1]) * ny + (pz - p[2]) * nz;
	t /= d[0] * nx + d[1] * ny + d[2] * nz;

	hit_x = p[0] + t * d[0];
	hit_y = p[1] + t * d[1];
	hit_z = p[2] + t * d[2];

	hit_x -= px;
	hit_y -= py;
	hit_z -= pz;
}

void PointSelector::select_z(double theta, double mouse_x, double mouse_y, double x, double y, double& z)
{
	double p[3];
	double d[3];
	double wx, wy;
	wx = mouse_x;
	wy = parent->h() - mouse_y;
	gluUnProject(wx, wy, 0, model, proj, vp, p, p + 1, p + 2);
	gluUnProject(wx, wy, 1, model, proj, vp, d, d + 1, d + 2);

	d[0] -= p[0];
	d[1] -= p[1];
	d[2] -= p[2];

	double tan_theta = tan(theta);

	double t = (x + y * tan_theta - p[1] * tan_theta - p[0]);
	t = t / (d[0] + d[1] * tan_theta);

	z = p[2] + t * d[2];

}

int PointSelector::process(int e)
{
	switch (e)
	{
	case FL_PUSH:
		{
			switch (Fl::event_button())
			{
			case FL_LEFT_MOUSE:
				double p[3];
				double d[3];
				double wx, wy;
				wx = Fl::event_x();
				wy = parent->h() - Fl::event_y();
				gluUnProject(wx, wy, 0, model, proj, vp, p, p + 1, p + 2);
				gluUnProject(wx, wy, 1, model, proj, vp, d, d + 1, d + 2);

				d[0] -= p[0];
				d[1] -= p[1];
				d[2] -= p[2];
				if (active)
					Config::instance().world()->select(p, d);
				return 1;
			}
		}
	case FL_MOVE:
		{
			double p[3];
			double d[3];
			double wx, wy;
			wx = Fl::event_x();
			wy = parent->h() - Fl::event_y();
			gluUnProject(wx, wy, 0, model, proj, vp, p, p + 1, p + 2);
			gluUnProject(wx, wy, 1, model, proj, vp, d, d + 1, d + 2);

			d[0] -= p[0];
			d[1] -= p[1];
			d[2] -= p[2];
			if (active)
				Config::instance().world()->hover(p, d);
			return 1;

		}
	}


}
