#include "normal_window.h"
#include "Config.h"
#include "Point.h"


#include <windows.h>

#include <FL/Fl.h>
#include <FL/Gl.h>

#include <iostream>
#include <cmath>

using namespace std;

normal_window::normal_window(int x, int y, int w, int h, char* t) : Fl_Gl_Window(x, y, w, h, t), 
	camera(50, 45, 10, -400, 400, 2, 30, this), selector(this), 
	cfg(Config::instance()) 
{FltkListener::add(&camera); FltkListener::add(&selector);selector.deactivate();}

normal_window::~normal_window(void)
{
}


void normal_window::draw()
{
	if (!valid())
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glViewport(0, 0, w(), h());		
	}

	glClearColor(1, 1, 1, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);

	glEnable (GL_CULL_FACE);
	glCullFace (GL_BACK);
	
	Point::triple focus = Config::instance().curve()->normal_pos();
	
	camera.focus_point(focus);

	camera.projection();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	selector.update();
	
	Config& cfg = Config::instance();

	cfg.force_normals = true;
	
	cfg.world()->draw();

	cfg.focus();

	cfg.force_normals = false;
}


int normal_window::process(int e)
{	
	switch (e)
	{
	case FL_FOCUS:
		return 1;
		
	case FL_UNFOCUS:
		return 1;
	


	case FL_DRAG:
		if (Fl::event_button() != FL_LEFT_MOUSE)
			return 0;
				
		double hit_x, hit_y, hit_z;
		Point::triple pos = cfg.curve()->normal_pos();
		Point::triple dir = cfg.curve()->normal_dir();
		selector.select_norm_dir(Fl::event_x(), Fl::event_y(), pos.x, pos.y, pos.z, 
			dir.x, dir.y, dir.z, hit_x, hit_y, hit_z);

		Point::triple up = Point::cross(Point::cross(dir, Point::triple(0,0,1)), dir);
		Point::triple out = dir.rotate(up, 90);
		Point::triple hit = Point::triple (hit_x, hit_y, hit_z);

		up.normalize();
		out.normalize();
		hit.normalize();

		double x, y;
		x = out.dot(hit);
		y = up.dot(hit);

		*cfg.curve()->normal() = atan2(y, x) * 180 / 3.1415927 - 90;

		return 1;
	}

	return Fl_Gl_Window::handle(e);
}